# -*- coding: utf-8 -*-
from nose.tools import assert_equal
from nose.tools import assert_true
from nose.tools import assert_false
import pygraphviz as pgv
import os
import tempfile

def test_readwrite():
    A = pgv.AGraph(name='test graph')
    A.add_path([1,2,3,4,5,6,7,8,9,10])
    the_file = tempfile.NamedTemporaryFile(delete=False)
    fname = the_file.name
    # Make sure it can be opened for writing again on Win32
    the_file.close()
    # Pass a string to trigger the code paths that close the newly created file handle
    A.write(fname)
    B = pgv.AGraph(fname)
    assert_equal(A, B)
    assert_true(B == A)
    assert_false(B is A)
    os.unlink(fname)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Setup commands for PyGraphviz.
"""
#    Copyright (C) 2006-2014 by
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Manos Renieris, http://www.cs.brown.edu/~er/
#    Distributed with BSD license.
#    All rights reserved, see LICENSE for details.
#
#   Derived from http://www.niteoweb.com/blog/setuptools-run-custom-code-during-install
#   Author: Maksym Markov <maksym.markov@gmail.com>

from setuptools.command.develop import develop
from setuptools.command.install import install

from setup_extra import get_graphviz_dirs

def add_extensions(command_subclass):
    """A decorator for classes subclassing one of the setuptools commands.

    It modifies the run() method so that it prints a friendly greeting.
    """
    orig_init = command_subclass.__init__
    orig_initialize_options = command_subclass.initialize_options
    orig_run = command_subclass.run
    command_subclass.user_options.extend([('include-path=', None, 'path to graphviz include files.'),
                                          ('library-path=', None, 'path to graphviz library files.')])

    def __init__(self, *args, **kws):
        orig_init(self, *args, **kws)

    def modified_initialize_options(self):
        self.include_path = None
        self.library_path = None
        orig_initialize_options(self)

    def modified_run(self):
        # Add extension here
        # if there is no library_path and include_path passed form command line then try identify them
        if (not self.include_path) or (not self.library_path):
            self.include_path, self.library_path = get_graphviz_dirs()
        if self.distribution and self.distribution.ext_modules:
            for m in self.distribution.ext_modules:
                if m.name == 'pygraphviz._graphviz':
                    if self.include_path:
                        m.include_dirs.append(self.include_path)
                    if self.library_path:
                        m.library_dirs.append(self.library_path)
        orig_run(self)

    command_subclass.__init__ = __init__
    command_subclass.initialize_options = modified_initialize_options
    command_subclass.run = modified_run

    return command_subclass


@add_extensions
class AddExtensionDevelopCommand(develop):
    pass

@add_extensions
class AddExtensionInstallCommand(install):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Setup helpers for PyGraphviz.
"""
#    Copyright (C) 2006-2015 by
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Manos Renieris, http://www.cs.brown.edu/~er/
#    Distributed with BSD license.     
#    All rights reserved, see LICENSE for details.

from __future__ import print_function
from __future__ import absolute_import
import subprocess as S
import sys
import os


def _b2str(buffer):
    result = u''
    if sys.version_info >= (3, 0):
        encoding = sys.getfilesystemencoding()
        if not encoding:
            # can be run without stdout
            if sys.stdout and sys.stdout.encoding:
                # encoding is not None only staring Python 3.2
                encoding = sys.stdout.encoding
            else:
                # fall back to default encoding ( normally it should not happen)
                encoding = 'utf8'
        if buffer:
            result = buffer.decode(encoding)
        return result
    else:
        # in Python 2 conversion is implicit
        return buffer


def _dpkg_config():
    # attempt to find graphviz installation with pkg-config
    # should work with modern versions of graphviz
    include_dirs=None
    library_dirs=None
    
    try:
        output = S.check_output(['dpkg', '-S', 'graphviz'])
        output = _b2str(output)
        lines = output.split('\n')
        for line in lines:
            if not include_dirs and line.endswith('.h'):
                include_dirs = os.path.dirname(line.split(':')[1].strip())
                include_dirs = include_dirs.strip() or None
            if not library_dirs and line.endswith('.so'):
                library_dirs = os.path.dirname(line.split(':')[1].strip())
                library_dirs = library_dirs.strip() or None
            if include_dirs and library_dirs:
                break
    except OSError:
        print("Failed to find dpkg")
    except S.CalledProcessError:
        print("Could not run dpkg")
    return include_dirs, library_dirs


def _pkg_config():
    # attempt to find graphviz installation with pkg-config
    # should work with modern versions of graphviz
    include_path=None
    library_path=None
    try:
        output = S.check_output(['pkg-config', '--libs-only-L', 'libcgraph'])
        output = _b2str(output)
        if output:
            library_path = output.strip()[2:]
            library_path = library_path.strip() or None
        output = S.check_output(['pkg-config', '--cflags-only-I', 'libcgraph'])
        output = _b2str(output)
        if output:
            include_path = output.strip()[2:]
            include_path = include_path.strip() or None
    except OSError:
        print("Failed to find pkg-config")
    return include_path, library_path

def _dotneato_config():
    # find graphviz configuration with dotneato-config
    # works with older versions of graphviz
    # attempt to find graphviz installation with pkg-config
    # should work with modern versions of graphviz
    library_path=None
    include_path=None
    try:
        output = S.check_output(['dotneato-config', '--ldflags', '--cflags'])
        output = _b2str(output)
        if output:
            include_path, library_path = output.split()
            library_path = library_path.strip()[2:].strip() or None
            include_path = include_path.strip()[2:].strip() or None
    except OSError:
        print("Failed to find dotneato-config")
        # fall through and test the other syntax
    if not include_path or not library_path:
        try:
            output = S.check_output(['dotneato-config', '--libs', '--cflags'])
            output = _b2str(output)
            if output:
                include_path, library_path = output.split('\n',1)
                library_path = library_path or library_path.strip()[2:].strip() or None
                include_path = include_path or include_path.strip()[2:].strip() or None
        except OSError:
            print("Failed to find dotneato-config")

    return include_path, library_path

def _try_configure(include_dirs, library_dirs, try_function):
    i, l = try_function()
    i = include_dirs or i
    l = library_dirs or l
    return i, l


def get_graphviz_dirs():
    """
    First try to read include_dirs from
    :return: tuple of lists ([include_dirs], [library_dirs])
    """

    # If the setup script couldn't find your graphviz installation you can
    # specify it here by uncommenting these lines or providing your own:
    # You must set both 'library_dirs' and 'include_dirs'

    # Linux, generic UNIX
    #library_dirs='/usr/lib/graphviz'
    #include_dirs='/usr/include/graphviz'

    # OSX, Linux, alternate location
    #library_dirs='/usr/local/lib/graphviz'
    #include_dirs='/usr/local/include/graphviz'

    # OSX (Fink)
    #library_dirs='/sw/lib/graphviz'
    #include_dirs='/sw/include/graphviz'

    # OSX (MacPorts)
    #library_dirs='/opt/local/lib/graphviz'
    #include_dirs='/opt/local/include/graphviz'

    # Windows
    # Unknown - use command line -I and -L switches to set
    include_dirs = None
    library_dirs = None

    if sys.platform != "win32":
        # Attempt to find Graphviz installation
        if library_dirs is None and include_dirs is None:
            print("Trying dpkg")
            include_dirs, library_dirs = _try_configure(include_dirs, library_dirs, _dpkg_config)

        if library_dirs is None and include_dirs is None:
            print("Trying pkg-config")
            include_dirs, library_dirs = _try_configure(include_dirs, library_dirs, _pkg_config)

        if library_dirs is None and include_dirs is None:
            print("Trying dotneato-config")
            include_dirs, library_dirs = _try_configure(include_dirs, library_dirs, _dotneato_config)

        if library_dirs is None and include_dirs is None:
            print()
            print("""Your Graphviz installation could not be found.

        1) You don't have Graphviz installed:
           Install Graphviz (http://graphviz.org)

        2) Your Graphviz package might incomplete.
           Install the binary development subpackage (e.g. libgraphviz-dev or similar.)

        3) You are using Windows
           There are no PyGraphviz binary packages for Windows but you might be
           able to build it from this source.  See
           http://networkx.lanl.gov/pygraphviz/reference/faq.html

        If you think your installation is correct you will need to manually
        provide path to graphviz include and library. For example:

        pip install pygraphviz --install-option="--include-path=/usr/include/graphviz" --install-option="--library-path=/usr/lib/graphviz/"

        The current setting of library_dirs and include_dirs is:""")
            print("library_dirs=%s"%library_dirs)
            print("include_dirs=%s"%include_dirs)
            print()
            raise OSError("Error locating graphviz.")

    print("include_dirs=%s" % include_dirs)
    print("library_dirs=%s" % library_dirs)


    return include_dirs, library_dirs




# -*- coding: utf-8 -*-
from nose.tools import *
import pygraphviz as pgv
from os import linesep

long_html_string = '''<<TABLE BORDER=0>
  <TR>
      <TD> meow </TD>
  </TR>
  <TR>
      <TD>
        <TABLE>
          <TR>
          <TD align=left>Count</TD>
          <TD align=right> 4 </TD>
          </TR>
          <TR>
          <TD align=left>Earliest Run</TD>
          <TD align=right> yesterday </TD>
          </TR>
          <TR>
          <TD align=left>Latest Run</TD>
          <TD align=right> tomorrow </TD>
          </TR>
          <TR>
          <TD align=left>Avg Runtime</TD>
          <TD align=right> 4 seconds </TD>
          </TR>
          <TR>
          <TD align=left>Avg Failure Rate</TD>
          <TD align=right> 38.1% </TD>
          </TR>
        </TABLE>
      </TD>
  </TR>
</TABLE>>'''

def test_long_html_string():
    G = pgv.AGraph(label='<Hello<BR/>Graph>')
    G.add_node('a', label=long_html_string)
    s = G.add_subgraph('b', label='<Hello<BR/>Subgraph>')
    s.add_node('sa', label='<Hello<BR/>Subgraph Node b>')
    G.add_edge('a','b', label='<Hello<BR/>Edge>')
    assert_equal.__self__.maxDiff = None
    assert_equal(G.string().expandtabs(2),
"""strict graph {{
  graph [label=<Hello<BR/>Graph>];
  node [label="\\N"];
  {{
    graph [label=<Hello<BR/>Subgraph>];
    sa     [label=<Hello<BR/>Subgraph Node b>];
  }}
  a  [label={0}];
  a -- b   [label=<Hello<BR/>Edge>];
}}
""".format(long_html_string).replace('\n', linesep))

def test_html():
    G = pgv.AGraph(label='<Hello<BR/>Graph>')
    G.add_node('a', label='<Hello<BR/>Node>')
    s = G.add_subgraph('b', label='<Hello<BR/>Subgraph>')
    s.add_node('sa', label='<Hello<BR/>Subgraph Node b>')
    G.add_edge('a','b', label='<Hello<BR/>Edge>')
    assert_equal(G.string().expandtabs(2),
"""strict graph {
  graph [label=<Hello<BR/>Graph>];
  node [label="\\N"];
  {
    graph [label=<Hello<BR/>Subgraph>];
    sa     [label=<Hello<BR/>Subgraph Node b>];
  }
  a  [label=<Hello<BR/>Node>];
  a -- b   [label=<Hello<BR/>Edge>];
}
""".replace('\n', linesep))



import sys
from nose.tools import *
from mock import patch

# inside distributed version, there is no setup_extra files already
sys.path.insert(0,'../..')
try:
    setup_extra = __import__('setup_extra')
except ImportError:
    setup_extra = None


def _get_values():
    return None, 'library'


def _get_dpkg_output():
    r = """libgvc: /usr/local/include/graphviz/gvc.h
                     gphviz: /usr/local/lib/graphviz/graphviz.so"""
    if sys.version_info >= (3, 0):
        s = bytes(r, encoding='utf8')
    else:
        s = str(r)
    return s


def test_try():
    if setup_extra:
        i, l = None, None
        i, l = setup_extra._try_configure(i, l, _get_values)
        assert_equals(i, None)
        assert_equals(l, 'library')


@patch('subprocess.check_output')
def test_dpkg_config(subprocess_mock):
    if setup_extra:
        subprocess_mock.return_value = _get_dpkg_output()
        i, l = setup_extra._dpkg_config()
        assert_equal(i, '/usr/local/include/graphviz')
        assert_equal(l, '/usr/local/lib/graphviz')


def test_b2str():
    if setup_extra:
        if sys.version_info >= (3, 0):
            r = bytes('abc', encoding='utf8')
        else:
            r = str('abc')
        s = setup_extra._b2str(r)
        assert_equal(s, u'abc')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Setup script for PyGraphviz
"""
from __future__ import absolute_import
from __future__ import print_function
from __future__ import division

from glob import glob

import os
from setuptools import setup, Extension
import sys

from setup_commands import AddExtensionDevelopCommand, AddExtensionInstallCommand
from setup_extra import get_graphviz_dirs


if os.path.exists('MANIFEST'): os.remove('MANIFEST')

if sys.argv[-1] == 'setup.py':
    print("To install, run 'python setup.py install'")
    print()

if sys.version_info[:2] < (2, 7):
    print("PyGraphviz requires Python version 2.7 or later (%d.%d detected)." %
          sys.version_info[:2])
    sys.exit(-1)

# Write the version information.
#TODO rework this import hack with import graphviz.release or import graphviz.version ( doesn't work now because of code in the __init__)
sys.path.insert(0, 'pygraphviz')
import release
release.write_versionfile()
sys.path.pop(0)

packages = ["pygraphviz", "pygraphviz.tests"]
docdirbase = 'share/doc/pygraphviz-%s' % release.version
data = [
    (docdirbase, glob("*.txt")),
    (os.path.join(docdirbase, 'examples'), glob("examples/*.py")),
    (os.path.join(docdirbase, 'examples'), glob("examples/*.dat")),
    (os.path.join(docdirbase, 'examples'), glob("examples/*.dat.gz")),
]
package_data = {'': ['*.txt'], }

if __name__ == "__main__":
    define_macros = []
    if sys.platform == "win32":
        define_macros = define_macros.append(('GVDLL', None))

    extension = [
        Extension(
            "pygraphviz._graphviz",
            ["pygraphviz/graphviz_wrap.c"],
            include_dirs=[],
            library_dirs=[],
            # cdt does not link to cgraph, whereas cgraph links to cdt.
            # thus, cdt needs to come first in the library list to be sure
            # that both libraries are linked in the final built .so (if cgraph
            # is first, the implicit inclusion of cdt can lead to an incomplete
            # link list, having only cdt and preventing the module from being loaded with
            # undefined symbol errors. seen under PyPy on Linux.)
            libraries=["cdt", "cgraph"],
            define_macros=define_macros
        )
    ]

    setup(
        name=release.name,
        version=release.version,
        author=release.authors['Hagberg'][0],
        author_email=release.authors['Hagberg'][1],
        description=release.description,
        keywords=release.keywords,
        long_description=release.long_description,
        license=release.license,
        platforms=release.platforms,
        url=release.url,
        download_url=release.download_url,
        classifiers=release.classifiers,
        packages=packages,
        data_files=data,
        ext_modules=extension,
        cmdclass={
            'install': AddExtensionInstallCommand,
            'develop': AddExtensionDevelopCommand,
            },
        package_data=package_data,
        include_package_data = True,
        test_suite='nose.collector',
        tests_require=['nose>=1.3.7', 'doctest-ignore-unicode>=0.1.2', 'mock>=2.0.0'],
    )



# -*- coding: utf-8 -*-
#
# Sphinx documentation build configuration file, created by
# sphinx-quickstart.py on Sat Mar  8 21:47:50 2008.
#
# This file is execfile()d with the current directory set to its containing dir.
#
# The contents of this file are pickled, so don't put values in the namespace
# that aren't pickleable (module imports are okay, they're removed automatically).
#
# All configuration values have a default value; values that are commented out
# serve to show the default value.

import sys, os, re
from datetime import date

# If your extensions are in another directory, add it here.
#sys.path.append(os.path.dirname(__file__))
sys.path.append(os.path.abspath('../sphinxext'))
#sys.path.append(os.path.abspath('../sphinxext/numpyext'))

# General configuration
# ---------------------

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.addons.*') or your custom ones.
extensions = ['sphinx.ext.autodoc', 'sphinx.ext.imgmath','sphinx.ext.doctest']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.                                      
source_encoding = 'utf-8'

# The master toctree document.
master_doc = 'index'

# General substitutions.
project = 'PyGraphviz'
copyright = '2004-{}, PyGraphviz Developers'.format(date.today().year)


# The default replacements for |version| and |release|, also used in various
# other places throughout the built documents.
#
# The short X.Y version.
import pygraphviz
version =pygraphviz.__version__
# The full version, including alpha/beta/rc tags.
release = version


# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
#today_fmt = '%B %d, %Y'

# List of documents that shouldn't be included in the build.
unused_docs = []

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = False

show_authors = True

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'


# Options for HTML output
# -----------------------

# The style sheet to use for HTML and HTML Help pages. A file of that name
# must exist either in Sphinx' static/ path, or in one of the custom paths
# given in html_static_path.
#html_style = 'sphinxdoc.css'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
#html_static_path = ['static']

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# Content template for the index page.
html_index = 'contents.html'

# Custom sidebar templates, maps page names to templates.
#html_sidebars = {'index': 'indexsidebar.html'}

# Additional templates that should be rendered to pages, maps page names to
# templates.
#html_additional_pages = {'index': 'index.html'}

# If true, the reST sources are included in the HTML build as _sources/<name>.
html_copy_source = False

html_use_opensearch = 'http://pygraphviz.github.io'

# Output file base name for HTML help builder.
htmlhelp_basename = 'PyGraphviz'

pngmath_use_preview = True

# Options for LaTeX output
# ------------------------

# The paper size ('letter' or 'a4').
latex_paper_size = 'letter'

# The font size ('10pt', '11pt' or '12pt').
#latex_font_size = '10pt'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, document class [howto/manual]).
latex_documents = [('index', 'pygraphviz.tex', 'PyGraphviz Documentation',
                    'PyGraphviz Developers', 'manual', 1)]


#latex_use_parts = True

# Additional stuff for the LaTeX preamble.
latex_elements = {
    'fontpkg': '\\usepackage{palatino}'
}

# Documents to append as an appendix to all manuals.
#latex_appendices = []


# Extension interface
# -------------------

from sphinx import addnodes

dir_sig_re = re.compile(r'\.\. ([^:]+)::(.*)$')

def parse_directive(env, sig, signode):
    if not sig.startswith('.'):
        dec_sig = '.. %s::' % sig
        signode += addnodes.desc_name(dec_sig, dec_sig)
        return sig
    m = dir_sig_re.match(sig)
    if not m:
        signode += addnodes.desc_name(sig, sig)
        return sig
    name, args = m.groups()
    dec_name = '.. %s::' % name
    signode += addnodes.desc_name(dec_name, dec_name)
    signode += addnodes.desc_addname(args, args)
    return name


def parse_role(env, sig, signode):
    signode += addnodes.desc_name(':%s:' % sig, ':%s:' % sig)
    return sig


event_sig_re = re.compile(r'([a-zA-Z-]+)\s*\((.*)\)')

def parse_event(env, sig, signode):
    m = event_sig_re.match(sig)
    if not m:
        signode += addnodes.desc_name(sig, sig)
        return sig
    name, args = m.groups()
    signode += addnodes.desc_name(name, name)
    plist = addnodes.desc_parameterlist()
    for arg in args.split(','):
        arg = arg.strip()
        plist += addnodes.desc_parameter(arg, arg)
    signode += plist
    return name


def setup(app):
    from sphinx.ext.autodoc import cut_lines
    app.connect('autodoc-process-docstring', cut_lines(4, what=['module']))
    app.add_description_unit('directive', 'dir', 'pair: %s; directive', parse_directive)
    app.add_description_unit('role', 'role', 'pair: %s; role', parse_role)
    app.add_description_unit('confval', 'confval', 'pair: %s; configuration value')
    app.add_description_unit('event', 'event', 'pair: %s; event', parse_event)



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.7
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.





from sys import version_info
if version_info >= (2, 6, 0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_graphviz', [dirname(__file__)])
        except ImportError:
            import _graphviz
            return _graphviz
        if fp is not None:
            try:
                _mod = imp.load_module('_graphviz', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _graphviz = swig_import_helper()
    del swig_import_helper
else:
    import _graphviz
del version_info
try:
    _swig_property = property
except NameError:
    pass  # Python < 2.2 doesn't have 'property'.


def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"):
        return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if (not static):
        if _newclass:
            object.__setattr__(self, name, value)
        else:
            self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr_nondynamic(self, class_type, name, static=1):
    if (name == "thisown"):
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    if (not static):
        return object.__getattr__(self, name)
    else:
        raise AttributeError(name)

def _swig_getattr(self, class_type, name):
    return _swig_getattr_nondynamic(self, class_type, name, 0)


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object:
        pass
    _newclass = 0



def agopen(name, kind, disc):
    return _graphviz.agopen(name, kind, disc)
agopen = _graphviz.agopen

def agraphnew(name,strict=False,directed=False):
    if strict:
       if directed:
            return _graphviz.agopen(name,cvar.Agstrictdirected,None)
       else:
            return _graphviz.agopen(name,cvar.Agstrictundirected,None)
    else:
        if directed:
            return _graphviz.agopen(name,cvar.Agdirected,None)
        else:		 
            return _graphviz.agopen(name,cvar.Agundirected,None)


def agclose(g):
    return _graphviz.agclose(g)
agclose = _graphviz.agclose

def agread(file, arg2):
    return _graphviz.agread(file, arg2)
agread = _graphviz.agread

def agwrite(g, file):
    return _graphviz.agwrite(g, file)
agwrite = _graphviz.agwrite

def agisundirected(g):
    return _graphviz.agisundirected(g)
agisundirected = _graphviz.agisundirected

def agisdirected(g):
    return _graphviz.agisdirected(g)
agisdirected = _graphviz.agisdirected

def agisstrict(g):
    return _graphviz.agisstrict(g)
agisstrict = _graphviz.agisstrict

def agnode(g, name, createflag):
    return _graphviz.agnode(g, name, createflag)
agnode = _graphviz.agnode

def agidnode(g, id, createflag):
    return _graphviz.agidnode(g, id, createflag)
agidnode = _graphviz.agidnode

def agsubnode(g, n, createflag):
    return _graphviz.agsubnode(g, n, createflag)
agsubnode = _graphviz.agsubnode

def agfstnode(g):
    return _graphviz.agfstnode(g)
agfstnode = _graphviz.agfstnode

def agnxtnode(g, n):
    return _graphviz.agnxtnode(g, n)
agnxtnode = _graphviz.agnxtnode

def aglstnode(g):
    return _graphviz.aglstnode(g)
aglstnode = _graphviz.aglstnode

def agprvnode(g, n):
    return _graphviz.agprvnode(g, n)
agprvnode = _graphviz.agprvnode

def agedge(g, t, h, name, createflag):
    return _graphviz.agedge(g, t, h, name, createflag)
agedge = _graphviz.agedge

def agidedge(g, t, h, id, createflag):
    return _graphviz.agidedge(g, t, h, id, createflag)
agidedge = _graphviz.agidedge

def agsubedge(g, e, createflag):
    return _graphviz.agsubedge(g, e, createflag)
agsubedge = _graphviz.agsubedge

def agfstin(g, n):
    return _graphviz.agfstin(g, n)
agfstin = _graphviz.agfstin

def agnxtin(g, e):
    return _graphviz.agnxtin(g, e)
agnxtin = _graphviz.agnxtin

def agfstout(g, n):
    return _graphviz.agfstout(g, n)
agfstout = _graphviz.agfstout

def agnxtout(g, e):
    return _graphviz.agnxtout(g, e)
agnxtout = _graphviz.agnxtout

def agfstedge(g, n):
    return _graphviz.agfstedge(g, n)
agfstedge = _graphviz.agfstedge

def agnxtedge(g, e, n):
    return _graphviz.agnxtedge(g, e, n)
agnxtedge = _graphviz.agnxtedge

def aghead(e):
    return _graphviz.aghead(e)
aghead = _graphviz.aghead

def agtail(e):
    return _graphviz.agtail(e)
agtail = _graphviz.agtail

def agattr(g, kind, name, value):
    return _graphviz.agattr(g, kind, name, value)
agattr = _graphviz.agattr

def agattrsym(obj, name):
    return _graphviz.agattrsym(obj, name)
agattrsym = _graphviz.agattrsym

def agnxtattr(g, kind, attr):
    return _graphviz.agnxtattr(g, kind, attr)
agnxtattr = _graphviz.agnxtattr

def agget(obj, name):
    return _graphviz.agget(obj, name)
agget = _graphviz.agget

def agxget(obj, sym):
    return _graphviz.agxget(obj, sym)
agxget = _graphviz.agxget

def agset(obj, name, value):
    return _graphviz.agset(obj, name, value)
agset = _graphviz.agset

def agxset(obj, sym, value):
    return _graphviz.agxset(obj, sym, value)
agxset = _graphviz.agxset

def agsafeset(obj, name, value, arg4):
    return _graphviz.agsafeset(obj, name, value, arg4)
agsafeset = _graphviz.agsafeset

def agattrname(atsym):
    return _graphviz.agattrname(atsym)
agattrname = _graphviz.agattrname

def agattrdefval(atsym):
    return _graphviz.agattrdefval(atsym)
agattrdefval = _graphviz.agattrdefval

def agsafeset_label(g, obj, name, val, arg5):
    return _graphviz.agsafeset_label(g, obj, name, val, arg5)
agsafeset_label = _graphviz.agsafeset_label

def agattr_label(g, kind, name, val):
    return _graphviz.agattr_label(g, kind, name, val)
agattr_label = _graphviz.agattr_label

def agsubg(g, name, createflag):
    return _graphviz.agsubg(g, name, createflag)
agsubg = _graphviz.agsubg

def agfstsubg(g):
    return _graphviz.agfstsubg(g)
agfstsubg = _graphviz.agfstsubg

def agnxtsubg(subg):
    return _graphviz.agnxtsubg(subg)
agnxtsubg = _graphviz.agnxtsubg

def agparent(g):
    return _graphviz.agparent(g)
agparent = _graphviz.agparent

def agroot(g):
    return _graphviz.agroot(g)
agroot = _graphviz.agroot

def agdelsubg(g, sub):
    return _graphviz.agdelsubg(g, sub)
agdelsubg = _graphviz.agdelsubg

def agnnodes(g):
    return _graphviz.agnnodes(g)
agnnodes = _graphviz.agnnodes

def agnedges(g):
    return _graphviz.agnedges(g)
agnedges = _graphviz.agnedges

def agdegree(g, n, arg3, out):
    return _graphviz.agdegree(g, n, arg3, out)
agdegree = _graphviz.agdegree

def agraphof(arg1):
    return _graphviz.agraphof(arg1)
agraphof = _graphviz.agraphof

def agnameof(arg1):
    return _graphviz.agnameof(arg1)
agnameof = _graphviz.agnameof

def agdelnode(g, arg_n):
    return _graphviz.agdelnode(g, arg_n)
agdelnode = _graphviz.agdelnode

def agdeledge(g, arg_e):
    return _graphviz.agdeledge(g, arg_e)
agdeledge = _graphviz.agdeledge

def agnameof(handle):
  name=_graphviz.agnameof(handle)
  if name is None:
     return None
  if name==b'' or name.startswith(b'%'):
    return None
  else:
    return name 


_graphviz.AGRAPH_swigconstant(_graphviz)
AGRAPH = _graphviz.AGRAPH

_graphviz.AGNODE_swigconstant(_graphviz)
AGNODE = _graphviz.AGNODE

_graphviz.AGOUTEDGE_swigconstant(_graphviz)
AGOUTEDGE = _graphviz.AGOUTEDGE

_graphviz.AGINEDGE_swigconstant(_graphviz)
AGINEDGE = _graphviz.AGINEDGE

_graphviz.AGEDGE_swigconstant(_graphviz)
AGEDGE = _graphviz.AGEDGE
# This file is compatible with both classic and new-style classes.

cvar = _graphviz.cvar
Agdirected = cvar.Agdirected
Agstrictdirected = cvar.Agstrictdirected
Agundirected = cvar.Agundirected
Agstrictundirected = cvar.Agstrictundirected

